#!/bin/bash

# Make sure 'ifup-routes' and 'ifup' have our handler provisions:
/usr/sausalito/constructor/base/network/10_fix_ifup.pl

# Make sure our IPs are all bound and our routes are all OK:
#/usr/sausalito/handlers/base/network/change_route.pl -c 2 >/dev/null 2>&1 || :

# Check if 'root-admin' password is empty. If so, get the hash of the 'root' password and set it:
PWPROBLEM=$(LC_ALL=C /usr/bin/passwd -S root-admin|/usr/bin/grep "Empty password"|/usr/bin/wc -l)
if [ "$PWPROBLEM" -eq "1" ];then
    /usr/bin/getent shadow|/usr/bin/grep root:|/usr/bin/cut -d : -f1-2|/usr/bin/sed "s@^root:@root-admin:@g"|/usr/sbin/chpasswd -e 
fi

# Remove double entries from /etc/crontab:
CRONISSUE=$(cat /etc/crontab|grep /etc/cron.quarter-daily|wc -l)
if [ "$CRONISSUE" -gt "1" ];then
    echo 'SHELL=/bin/bash' > /etc/crontab.new
    echo 'PATH=/sbin:/bin:/usr/sbin:/usr/bin' >> /etc/crontab.new
    echo 'MAILTO=root' >> /etc/crontab.new
    echo '' >> /etc/crontab.new
    echo '# For details see man 4 crontabs' >> /etc/crontab.new
    echo '' >> /etc/crontab.new
    echo '# Example of job definition:' >> /etc/crontab.new
    echo '# .---------------- minute (0 - 59)' >> /etc/crontab.new
    echo '# |  .------------- hour (0 - 23)' >> /etc/crontab.new
    echo '# |  |  .---------- day of month (1 - 31)' >> /etc/crontab.new
    echo '# |  |  |  .------- month (1 - 12) OR jan,feb,mar,apr ...' >> /etc/crontab.new
    echo '# |  |  |  |  .---- day of week (0 - 6) (Sunday=0 or 7) OR sun,mon,tue,wed,thu,fri,sat' >> /etc/crontab.new
    echo '# |  |  |  |  |' >> /etc/crontab.new
    echo '# *  *  *  *  * user-name  command to be executed' >> /etc/crontab.new
    echo '' >> /etc/crontab.new
    cat /etc/crontab|grep -v ^#|grep -e ^[0-9]|sort -u >> /etc/crontab.new
    echo '' >> /etc/crontab.new
    mv /etc/crontab.new /etc/crontab
    systemctl restart crond
fi

# Fix PHP Session dir GID if needed:
if [ -d "/var/lib/php/session" ];then
    SESSPERMS=`ls -la /var/lib/php|grep session|awk '{print $1}'`
    if [ $SESSPERMS != "drwxrwxrwx" ];then
        chmod 777 /var/lib/php/session
    fi
fi

# Fix potential license directory issues:
if [ ! -d "/usr/sausalito/license" ];then
    mkdir -p /usr/sausalito/license
    chmod 0700 /usr/sausalito/license
    chown admserv:admserv /usr/sausalito/license
else
    LICOWNER=`ls -la /usr/sausalito/|grep license|grep admserv|wc -l`
    if [ $LICOWNER -eq 0 ];then
        chmod 0700 /usr/sausalito/license
        chown admserv:admserv /usr/sausalito/license
    fi
fi

# Fix potential capcache directory issues:
if [ ! -d "/usr/sausalito/capcache" ];then
    mkdir -p /usr/sausalito/capcache
    chmod 0700 /usr/sausalito/capcache
    chown admserv:admserv /usr/sausalito/capcache
else
    CAPOWNER=`ls -la /usr/sausalito/|grep capcache|grep admserv|wc -l`
    if [ $CAPOWNER -eq 0 ];then
        chmod 0700 /usr/sausalito/capcache
        chown admserv:admserv /usr/sausalito/capcache
    fi
fi

# Remove the OS supplied YUM autoupdater (if present), as we bring our own:
if [ -f /etc/cron.daily/yum-autoupdate ] ; then
    rm -f /etc/cron.daily/yum-autoupdate
fi

# The CD-Installer of BlueOnyx brings /usr/bin/fix-httpd-log-dir aboard, but
# it might not be executable:
if [ -f /usr/bin/fix-httpd-log-dir ]; then
    chmod 755 /usr/bin/fix-httpd-log-dir
fi

# While we are at it, delete the default CentOS welcome page:
if [ -f /etc/httpd/conf.d/welcome.conf ]; then
        /bin/rm -f /etc/httpd/conf.d/welcome.conf
fi

# Also delete /etc/httpd/conf.d/manual.conf:
if [ -f /etc/httpd/conf.d/manual.conf ]; then
        /bin/rm -f /etc/httpd/conf.d/manual.conf
fi

# Also remove server.conf
if [ -f /etc/httpd/conf.d/server.conf ]; then
        /bin/rm -f /etc/httpd/conf.d/server.conf
fi

# Fix nss.conf if present:
if [ -f /etc/httpd/conf.d/nss.conf ]; then
NUMNSS=`/bin/cat /etc/httpd/conf.d/nss.conf | /bin/grep NSSEnforceValidCerts | /usr/bin/wc -l`
    if [ $NUMNSS = "0" ]; then
        sed -i "s/^NSSSession3CacheTimeout 86400/NSSSession3CacheTimeout 86400\\nNSSEnforceValidCerts off/g" /etc/httpd/conf.d/nss.conf
    fi
fi

# Need to turn off Chronyd, as we use NTPd:
if [ -f /usr/lib/systemd/system/chronyd.service ];then
    CHRONY=`LC_ALL=C systemctl status chronyd|grep "Active: active (running)"|wc -l`
    if [ $CHRONY -eq "1" ]; then
        systemctl stop chronyd &>/dev/null || :
        systemctl disable chronyd &>/dev/null || :
    fi
fi

# CentOS and their bloody appstreams break /etc/httpd/conf.d/php.conf.
PHPBROKEN=`/bin/cat /etc/httpd/conf.d/php.conf | /bin/grep "#    Require all denied" | /usr/bin/wc -l`
if [ $PHPBROKEN = "0" ]; then
    perl -0777 -i.bak -pe 's/#<Files ".user.ini">\n    Require all denied\n#<\/Files>/#<Files ".user.ini">\n#    Require all denied\n#<\/Files>/igs' /etc/httpd/conf.d/php.conf
fi

# Really, CentOS? I think you forgot something!
if [ ! -d /run/php-fpm ];then
    mkdir -p /run/php-fpm
fi

# Run Passive Monitor Support Account watcher (if present):
if [ -f /usr/sausalito/swatch/bin/am_support.pl ]; then
        /usr/sausalito/swatch/bin/am_support.pl
fi

# Fix DBUS issue if /var/run/dbus/system_bus_socket no longer is a Symlink:
#if [ -f /usr/bin/systemctl ];then
#    if [ ! -L /var/run/dbus/system_bus_socket ];then
#        systemctl stop dbus
#        rm /var/run/dbus/system_bus_socket
#        ln -s /run/dbus/system_bus_socket /var/run/dbus/system_bus_socket
#        systemctl start dbus
#    fi
#fi

# Fix Imagetragick issue:
if [ -f /etc/ImageMagick/policy.xml ];then
    ITRAGIC=`cat /etc/ImageMagick/policy.xml |grep TEXT|wc -l`
    # We need to apply the fix:
    if [ $ITRAGIC -eq 0 ]; then
        # Do we have a good copy? We should!
        if [ -f /etc/ImageMagick/policy.xml.fixed ];then
            cp /etc/ImageMagick/policy.xml.fixed /etc/ImageMagick/policy.xml
        fi
    fi
fi

# CentOS Snafu-fix:
if [ -f /usr/lib/systemd/system/auditd.service ];then
    AUDITD=`ls -la /usr/lib/systemd/system/auditd.service|grep "^-rw-r-----"|wc -l`
    if [ $AUDITD -eq 1 ]; then
        chmod 644 /usr/lib/systemd/system/auditd.service
    fi
fi

# Create Dovecot SSL parameters (DH-stuff) if it's missing:
if [ ! -f /var/lib/dovecot/ssl-parameters.dat ]; then
    /usr/libexec/dovecot/ssl-params &>/dev/null
fi

#
### Postfix related:
#

POSTFIXRESTART=0

# Fix alias location in /usr/libexec/postfix/aliasesdb:
if [ -f /usr/libexec/postfix/aliasesdb ];then
    POSTALIAS=`cat /usr/libexec/postfix/aliasesdb |grep /etc/mail/aliases|wc -l`
    # We need to apply the fix:
    if [ $POSTALIAS -eq 0 ]; then
        sed -i -e 's@/etc/aliases@/etc/mail/aliases@g' /usr/libexec/postfix/aliasesdb
        POSTFIXRESTART=1
    fi
fi

# Fix Postfix Unit-File:
if [ -f /usr/lib/systemd/system/postfix.service ];then
    POSTUNIT=`cat /usr/lib/systemd/system/postfix.service |grep blueonyx-postfix|wc -l`
    # We need to apply the fix:
    if [ $POSTUNIT -eq 0 ]; then
        echo '[Unit]' > /usr/lib/systemd/system/postfix.service
        echo 'Description=Postfix Mail Transport Agent' >> /usr/lib/systemd/system/postfix.service
        echo 'After=syslog.target network.target' >> /usr/lib/systemd/system/postfix.service
        echo 'Conflicts=sendmail.service exim.service' >> /usr/lib/systemd/system/postfix.service
        echo '' >> /usr/lib/systemd/system/postfix.service
        echo '[Service]' >> /usr/lib/systemd/system/postfix.service
        echo 'Type=forking' >> /usr/lib/systemd/system/postfix.service
        echo 'PIDFile=/var/spool/postfix/pid/master.pid' >> /usr/lib/systemd/system/postfix.service
        echo 'EnvironmentFile=-/etc/sysconfig/network' >> /usr/lib/systemd/system/postfix.service
        echo 'PrivateTmp=true' >> /usr/lib/systemd/system/postfix.service
        echo 'CapabilityBoundingSet=~ CAP_NET_ADMIN CAP_SYS_ADMIN CAP_SYS_BOOT CAP_SYS_MODULE' >> /usr/lib/systemd/system/postfix.service
        echo 'ProtectSystem=true' >> /usr/lib/systemd/system/postfix.service
        echo 'PrivateDevices=true' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecStartPre=-/usr/sausalito/bin/blueonyx-postfix' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecStartPre=-/usr/libexec/postfix/aliasesdb' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecStartPre=-/usr/libexec/postfix/chroot-update' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecStart=/usr/sbin/postfix start' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecReload=/usr/sbin/postfix reload' >> /usr/lib/systemd/system/postfix.service
        echo 'ExecStop=/usr/sbin/postfix stop' >> /usr/lib/systemd/system/postfix.service
        echo '' >> /usr/lib/systemd/system/postfix.service
        echo '[Install]' >> /usr/lib/systemd/system/postfix.service
        echo 'WantedBy=multi-user.target' >> /usr/lib/systemd/system/postfix.service
        echo '' >> /usr/lib/systemd/system/postfix.service
        systemctl daemon-reload
        POSTFIXRESTART=1
    fi
fi

# If Postfix is enabled check if we need to restart it:
if [ -f /etc/sysconfig/bxmta ];then
    POSTFIXENABLED=`cat /etc/sysconfig/bxmta|grep ^MTA=POSTFIX|wc -l`
    if [ $POSTFIXENABLED = "1" ]; then
        if [ "$POSTFIXRESTART" = "1" ];then
            systemctl restart postfix.service
        fi
    fi
fi

# If Apache has /etc/httpd/conf.modules.d/10-h2.conf, then we disable the module load:
if [ -f /etc/httpd/conf.modules.d/10-h2.conf ];then
    POSTPATCHE=`cat /etc/httpd/conf.modules.d/10-h2.conf |grep '^LoadModule http2_module'|wc -l`
    if [ $POSTPATCHE -eq 1 ]; then
        sed -i -e 's@^LoadModule http2_module modules/mod_http2.so@#LoadModule http2_module modules/mod_http2.so@g' /etc/httpd/conf.modules.d/10-h2.conf
        systemctl restart httpd >/dev/null 2>&1 || :
    fi
fi

# Epel:
# 
# People don't learn. If Epel is present, we disable it. Am sick of fixing broken boxes:
#
if [ -f /etc/yum.repos.d/epel.repo ];then
    sed -i -e 's|enabled=1|enabled=0|' /etc/yum.repos.d/epel.repo
fi

# Mitigation for Perl issues:
PERLBROKEN=`/bin/cat /etc/httpd/conf/mime.types /etc/mime.types | /bin/grep "^application/x-perl" | /usr/bin/wc -l`
if [ $PERLBROKEN != "0" ]; then
     sed -i -e 's@^application/x-perl.*$@#application/x-perl                              pl@g' /etc/httpd/conf/mime.types
     sed -i -e 's@^application/x-perl.*$@#application/x-perl                              pl@g' /etc/mime.types
fi

# Fix PID location in Sendmail Unit File
SENDMAILPID=$(cat /usr/lib/systemd/system/sendmail.service |grep '^PIDFile=/var/run/sendmail.pid'|wc -l)
if [ "$SENDMAILPID" -eq '1' ];then
    # Sendmail Systemd Unit-File has old style PID setting. Fix it!
    sed -i 's|^PIDFile=/var/run/sendmail.pid|PIDFile=/run/sendmail.pid|g' /usr/lib/systemd/system/sendmail.service
    /usr/bin/systemctl daemon-reload &>/dev/null || :
fi

# Start: YUM/DNF Wrapper
if [ -L "/usr/bin/yum" ] && [ -f "/usr/bin/yum_wrapper" ]; then
    yum_link=$(LC_ALL=C stat /usr/bin/yum | grep File: | awk '{print $4}')
    if [ "$yum_link" = "dnf-3" ]; then
        rm /usr/bin/yum
        ln -s /usr/bin/yum_wrapper /usr/bin/yum
    fi
fi

if [ -L "/usr/bin/dnf" ] && [ -f "/usr/bin/dnf_wrapper" ]; then
    dnf_link=$(LC_ALL=C stat /usr/bin/dnf | grep File: | awk '{print $4}')
    if [ "$dnf_link" = "dnf-3" ]; then
        rm /usr/bin/dnf
        ln -s /usr/bin/dnf_wrapper /usr/bin/dnf
    fi
fi
# End: YUM/DNF Wrapper

# Fix permissions for Mailman archive dir to allow GUI access:
if [ -d /var/lib/mailman/archives/private ];then
    MMARCHIVEDIR=$(ls -la /var/lib/mailman/archives/|grep private|grep -v "^drwxrwsr-x"|wc -l)
    if [ "$MMARCHIVEDIR" -eq '1' ];then
        # Wrong permissions. Fix them:
        chmod 2775 /var/lib/mailman/archives/private
    fi
fi

#
### Post Elmer-Install setup stuff:
#

if [ ! -f "/usr/lib/systemd/system/node_exporter.service" ] || [ ! -f "/usr/lib/systemd/system/prometheus.service" ]; then
    if [ -f /usr/sausalito/configs/prometheus/activate_prometheus.sh ];then
        /usr/sausalito/configs/prometheus/activate_prometheus.sh &>/dev/null || :
    fi
fi

# Directory permissions/ownership assurance for base-alpine dirs:

if [ ! -d /usr/sausalito/license ];then 
  mkdir /usr/sausalito/license 
fi

if [ -d /usr/sausalito/license ];then 
  chmod 700 /usr/sausalito/license/ 
  chown admserv:admserv /usr/sausalito/license
fi

if [ ! -d /usr/sausalito/capcache ];then 
    mkdir /usr/sausalito/capcache 
fi
if [ -d /usr/sausalito/capcache ];then 
    chmod 700 /usr/sausalito/capcache/ 
    chown admserv:admserv /usr/sausalito/capcache
fi

if [ ! -d /usr/sausalito/sessions ];then 
    mkdir /usr/sausalito/sessions 
fi

if [ -d /usr/sausalito/sessions ];then
    chmod 700 /usr/sausalito/sessions/ 
    chown admserv:admserv /usr/sausalito/sessions
fi

if [ -d /usr/sausalito/ui/chorizo/ci4/writable ];then
  chown -R admserv:admserv /usr/sausalito/ui/chorizo/ci4/writable
fi

if [ ! -f /var/log/gui-debug.log ];then
  touch /var/log/gui-debug.log
fi

if [ -f /var/log/gui-debug.log ];then
  chown admserv:admserv /var/log/gui-debug.log
fi

# Fix /etc/httpd/conf.d/blueonyx.conf if needed (remove /login redirects):
CONF_FILE="/etc/httpd/conf.d/blueonyx.conf"
if grep -q "^Rewrite" "$CONF_FILE"; then
    # Create backup:
    cp "$CONF_FILE" "$CONF_FILE.bak"
    sed -i '/^Rewrite/d' "$CONF_FILE"
    systemctl reload httpd &>/dev/null || :
fi

# Fix /etc/httpd/conf/vhosts/preview if needed (remove /login redirects):
PRECONF_FILE="/etc/httpd/conf/vhosts/preview"
if [ -f /etc/httpd/conf/vhosts/preview ];then
    if grep -q "^Rewrite" "$PRECONF_FILE"; then
        # Create backup:
        cp "$PRECONF_FILE" "$PRECONF_FILE.bak"
        sed -i '/^Rewrite/d' "$PRECONF_FILE"
        systemctl reload httpd &>/dev/null || :
    fi
fi

#
### Last rites before exit:
#

# Check if restart or rehash files exists and are older than 38 minutes
restart_file="/usr/sausalito/yumcce/restart"
rehash_file="/usr/sausalito/yumcce/rehash"
if [ -f "$restart_file" ] && [ $(find "$restart_file" -mmin +38) ]; then
    echo "Restart file exists and is older than 38 minutes"
    # Restart CCEd:
    /usr/sausalito/sbin/cced.init restart
    rm -f /usr/sausalito/yumcce/restart
    rm -f /usr/sausalito/yumcce/rehash
elif [ -f "$rehash_file" ] && [ $(find "$rehash_file" -mmin +38) ]; then
    echo "Rehash file exists and is older than 38 minutes"
    # Rehash CCEd:
    /usr/sausalito/sbin/cced.init rehash
    rm -f /usr/sausalito/yumcce/rehash
fi

# End:
exit

